-- MANIPULATING DATA

-- DML COMMANDS 
-- TRANSACTIONS


--1) INSERTING DATA IN A TABLE

-- SYNTAX: INSERT INTO <tables> VALUES (<columns>)
-- EXAMPLES:

DESC DEPARTMENTS;

INSERT INTO DEPARTMENTS(DEPARTMENT_ID
                        ,DEPARTMENT_NAME
                        ,MANAGER_ID
                        ,LOCATION_ID)
VALUES(300,'Maintainace',100,1700);

SELECT * FROM DEPARTMENTS;

INSERT INTO DEPARTMENTS(DEPARTMENT_ID,DEPARTMENT_NAME)
VALUES(310,'General Services');

SELECT * FROM DEPARTMENTS;

INSERT INTO DEPARTMENTS VALUES(320,'Research and Development',NULL,NULL);

SELECT * FROM DEPARTMENTS;

DESC EMPLOYEES;

INSERT INTO EMPLOYEES(EMPLOYEE_ID
                      ,FIRST_NAME
                      ,LAST_NAME
                      ,EMAIL
                      ,PHONE_NUMBER
                      ,HIRE_DATE
                      ,JOB_ID
                      ,SALARY
                      ,COMMISSION_PCT
                      ,MANAGER_ID
                      ,DEPARTMENT_ID)
VALUES(250,'John','Doe','DOE','515.124.4567',SYSDATE,'AC_ACCOUNT',6900
       ,NULL
       ,205
       ,100);                      

SELECT *
FROM EMPLOYEES
WHERE EMPLOYEE_ID = 250;

-- USING EXPLICT DEFAULT VALUES

INSERT INTO DEPARTMENTS(DEPARTMENT_ID, DEPARTMENT_NAME, MANAGER_ID)
VALUES(400,'Engineering',DEFAULT);

SELECT * FROM DEPARTMENTS;

--2) CHANGING DATA IN A TABLE

-- SYNTAX: UPDATE <table> SET <column=value, column=value...> WHERE <condition>

-- EXAMPLES: 

SELECT *
FROM EMPLOYEES
WHERE EMPLOYEE_ID = 114;

UPDATE EMPLOYEES
SET LAST_NAME = 'Raphael'
WHERE EMPLOYEE_ID = 114;

-- DO NOT FORGET THE WHERE CLAUSE!

-- USING SUBQUERIES IN AN UPDATE STATEMENT

UPDATE EMPLOYEES
SET JOB_ID = (SELECT JOB_ID
              FROM EMPLOYEES
              WHERE EMPLOYEE_ID = 205),
     SALARY = (SELECT SALARY
               FROM EMPLOYEES
               WHERE EMPLOYEE_ID = 205)
WHERE EMPLOYEE_ID = 115;
                        
SELECT *
FROM EMPLOYEES
WHERE EMPLOYEE_ID = 115 OR EMPLOYEE_ID = 115;
 
-- INTEGRITY CONSTRAINT ERROR

UPDATE EMPLOYEES
SET DEPARTMENT_ID = 55
WHERE DEPARTMENT_ID = 110;

SELECT * FROM DEPARTMENTS;

-- USING EXPLICT DEFAULT VALUES

UPDATE DEPARTMENTS
SET MANAGER_ID = DEFAULT WHERE DEPARTMENT_ID = 10;


--3) REMOVING A ROW FROM A TABLE

-- SYNTAX: DELETE [FROM] <table> WHERE <condition>

SELECT *
FROM DEPARTMENTS;

DELETE FROM DEPARTMENTS
WHERE DEPARTMENT_NAME = 'Maintainace'

-- DO NOT FORGET THE WHERE CLAUSE!

-- USING SUBQUERIES

DELETE FROM EMPLOYEES
WHERE DEPARTMENT_ID  = (SELECT DEPARTMENT_ID
                         FROM DEPARTMENTS
                         WHERE DEPARTMENT_NAME LIKE '%General Services%');
                         
                         
-- INTEGRITY CONSTRAINT ERROR

DELETE FROM DEPARTMENTS
WHERE DEPARTMENT_ID = 60;                         


-- TRANSACTIONS
-- BEGIN WITH A DML SQL STATEMENT. FINISHES WITH COMMIT OR ROLLBACK
-- ADVANTAGES: DATA CONSISTENCY, PREVIEW DATA, ACID PROPERTIES
-- A TRANSACTION CAN HAVE SAVEPOINTS:

UPDATE...
SAVEPOINT SAVE_1
INSERT...
ROLLBACK TO SAVE_1
COMMIT

-- EXAMPLE:

INSERT INTO DEPARTMENTS
VALUES(350,'Customer Support', NULL, 1700);
COMMIT;

SELECT * FROM DEPARTMENTS;
SELECT * FROM COUNTRIES;

INSERT INTO COUNTRIES VALUES('XX','Transilvania',2);
ROLLBACK;

-- READ CONSISTENCY
-- LOCKING